//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"context"
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/appcomplianceautomation/armappcomplianceautomation"
	"net/http"
	"net/url"
	"regexp"
)

// ScopingConfigurationServer is a fake server for instances of the armappcomplianceautomation.ScopingConfigurationClient type.
type ScopingConfigurationServer struct {
	// CreateOrUpdate is the fake for method ScopingConfigurationClient.CreateOrUpdate
	// HTTP status codes to indicate success: http.StatusOK, http.StatusCreated
	CreateOrUpdate func(ctx context.Context, reportName string, scopingConfigurationName string, properties armappcomplianceautomation.ScopingConfigurationResource, options *armappcomplianceautomation.ScopingConfigurationClientCreateOrUpdateOptions) (resp azfake.Responder[armappcomplianceautomation.ScopingConfigurationClientCreateOrUpdateResponse], errResp azfake.ErrorResponder)

	// Delete is the fake for method ScopingConfigurationClient.Delete
	// HTTP status codes to indicate success: http.StatusOK, http.StatusNoContent
	Delete func(ctx context.Context, reportName string, scopingConfigurationName string, options *armappcomplianceautomation.ScopingConfigurationClientDeleteOptions) (resp azfake.Responder[armappcomplianceautomation.ScopingConfigurationClientDeleteResponse], errResp azfake.ErrorResponder)

	// Get is the fake for method ScopingConfigurationClient.Get
	// HTTP status codes to indicate success: http.StatusOK
	Get func(ctx context.Context, reportName string, scopingConfigurationName string, options *armappcomplianceautomation.ScopingConfigurationClientGetOptions) (resp azfake.Responder[armappcomplianceautomation.ScopingConfigurationClientGetResponse], errResp azfake.ErrorResponder)

	// NewListPager is the fake for method ScopingConfigurationClient.NewListPager
	// HTTP status codes to indicate success: http.StatusOK
	NewListPager func(reportName string, options *armappcomplianceautomation.ScopingConfigurationClientListOptions) (resp azfake.PagerResponder[armappcomplianceautomation.ScopingConfigurationClientListResponse])
}

// NewScopingConfigurationServerTransport creates a new instance of ScopingConfigurationServerTransport with the provided implementation.
// The returned ScopingConfigurationServerTransport instance is connected to an instance of armappcomplianceautomation.ScopingConfigurationClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewScopingConfigurationServerTransport(srv *ScopingConfigurationServer) *ScopingConfigurationServerTransport {
	return &ScopingConfigurationServerTransport{
		srv:          srv,
		newListPager: newTracker[azfake.PagerResponder[armappcomplianceautomation.ScopingConfigurationClientListResponse]](),
	}
}

// ScopingConfigurationServerTransport connects instances of armappcomplianceautomation.ScopingConfigurationClient to instances of ScopingConfigurationServer.
// Don't use this type directly, use NewScopingConfigurationServerTransport instead.
type ScopingConfigurationServerTransport struct {
	srv          *ScopingConfigurationServer
	newListPager *tracker[azfake.PagerResponder[armappcomplianceautomation.ScopingConfigurationClientListResponse]]
}

// Do implements the policy.Transporter interface for ScopingConfigurationServerTransport.
func (s *ScopingConfigurationServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "ScopingConfigurationClient.CreateOrUpdate":
		resp, err = s.dispatchCreateOrUpdate(req)
	case "ScopingConfigurationClient.Delete":
		resp, err = s.dispatchDelete(req)
	case "ScopingConfigurationClient.Get":
		resp, err = s.dispatchGet(req)
	case "ScopingConfigurationClient.NewListPager":
		resp, err = s.dispatchNewListPager(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (s *ScopingConfigurationServerTransport) dispatchCreateOrUpdate(req *http.Request) (*http.Response, error) {
	if s.srv.CreateOrUpdate == nil {
		return nil, &nonRetriableError{errors.New("fake for method CreateOrUpdate not implemented")}
	}
	const regexStr = `/providers/Microsoft\.AppComplianceAutomation/reports/(?P<reportName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/scopingConfigurations/(?P<scopingConfigurationName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 2 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	body, err := server.UnmarshalRequestAsJSON[armappcomplianceautomation.ScopingConfigurationResource](req)
	if err != nil {
		return nil, err
	}
	reportNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("reportName")])
	if err != nil {
		return nil, err
	}
	scopingConfigurationNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("scopingConfigurationName")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := s.srv.CreateOrUpdate(req.Context(), reportNameParam, scopingConfigurationNameParam, body, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK, http.StatusCreated}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK, http.StatusCreated", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).ScopingConfigurationResource, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (s *ScopingConfigurationServerTransport) dispatchDelete(req *http.Request) (*http.Response, error) {
	if s.srv.Delete == nil {
		return nil, &nonRetriableError{errors.New("fake for method Delete not implemented")}
	}
	const regexStr = `/providers/Microsoft\.AppComplianceAutomation/reports/(?P<reportName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/scopingConfigurations/(?P<scopingConfigurationName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 2 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	reportNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("reportName")])
	if err != nil {
		return nil, err
	}
	scopingConfigurationNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("scopingConfigurationName")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := s.srv.Delete(req.Context(), reportNameParam, scopingConfigurationNameParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK, http.StatusNoContent}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK, http.StatusNoContent", respContent.HTTPStatus)}
	}
	resp, err := server.NewResponse(respContent, req, nil)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (s *ScopingConfigurationServerTransport) dispatchGet(req *http.Request) (*http.Response, error) {
	if s.srv.Get == nil {
		return nil, &nonRetriableError{errors.New("fake for method Get not implemented")}
	}
	const regexStr = `/providers/Microsoft\.AppComplianceAutomation/reports/(?P<reportName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/scopingConfigurations/(?P<scopingConfigurationName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 2 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	reportNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("reportName")])
	if err != nil {
		return nil, err
	}
	scopingConfigurationNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("scopingConfigurationName")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := s.srv.Get(req.Context(), reportNameParam, scopingConfigurationNameParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).ScopingConfigurationResource, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (s *ScopingConfigurationServerTransport) dispatchNewListPager(req *http.Request) (*http.Response, error) {
	if s.srv.NewListPager == nil {
		return nil, &nonRetriableError{errors.New("fake for method NewListPager not implemented")}
	}
	newListPager := s.newListPager.get(req)
	if newListPager == nil {
		const regexStr = `/providers/Microsoft\.AppComplianceAutomation/reports/(?P<reportName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/scopingConfigurations`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 1 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		reportNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("reportName")])
		if err != nil {
			return nil, err
		}
		resp := s.srv.NewListPager(reportNameParam, nil)
		newListPager = &resp
		s.newListPager.add(req, newListPager)
		server.PagerResponderInjectNextLinks(newListPager, req, func(page *armappcomplianceautomation.ScopingConfigurationClientListResponse, createLink func() string) {
			page.NextLink = to.Ptr(createLink())
		})
	}
	resp, err := server.PagerResponderNext(newListPager, req)
	if err != nil {
		return nil, err
	}
	if !contains([]int{http.StatusOK}, resp.StatusCode) {
		s.newListPager.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", resp.StatusCode)}
	}
	if !server.PagerResponderMore(newListPager) {
		s.newListPager.remove(req)
	}
	return resp, nil
}
