// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) Go Code Generator. DO NOT EDIT.

package fake

import (
	"errors"
	"fmt"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"strings"
	"sync"
)

// ServerFactory is a fake server for instances of the armconnectedcache.ClientFactory type.
type ServerFactory struct {
	// CacheNodesOperationsServer contains the fakes for client CacheNodesOperationsClient
	CacheNodesOperationsServer CacheNodesOperationsServer

	// EnterpriseCustomerOperationsServer contains the fakes for client EnterpriseCustomerOperationsClient
	EnterpriseCustomerOperationsServer EnterpriseCustomerOperationsServer

	// EnterpriseMccCacheNodesOperationsServer contains the fakes for client EnterpriseMccCacheNodesOperationsClient
	EnterpriseMccCacheNodesOperationsServer EnterpriseMccCacheNodesOperationsServer

	// EnterpriseMccCustomersServer contains the fakes for client EnterpriseMccCustomersClient
	EnterpriseMccCustomersServer EnterpriseMccCustomersServer

	// IspCacheNodesOperationsServer contains the fakes for client IspCacheNodesOperationsClient
	IspCacheNodesOperationsServer IspCacheNodesOperationsServer

	// IspCustomersServer contains the fakes for client IspCustomersClient
	IspCustomersServer IspCustomersServer

	// OperationsServer contains the fakes for client OperationsClient
	OperationsServer OperationsServer
}

// NewServerFactoryTransport creates a new instance of ServerFactoryTransport with the provided implementation.
// The returned ServerFactoryTransport instance is connected to an instance of armconnectedcache.ClientFactory via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewServerFactoryTransport(srv *ServerFactory) *ServerFactoryTransport {
	return &ServerFactoryTransport{
		srv: srv,
	}
}

// ServerFactoryTransport connects instances of armconnectedcache.ClientFactory to instances of ServerFactory.
// Don't use this type directly, use NewServerFactoryTransport instead.
type ServerFactoryTransport struct {
	srv                                       *ServerFactory
	trMu                                      sync.Mutex
	trCacheNodesOperationsServer              *CacheNodesOperationsServerTransport
	trEnterpriseCustomerOperationsServer      *EnterpriseCustomerOperationsServerTransport
	trEnterpriseMccCacheNodesOperationsServer *EnterpriseMccCacheNodesOperationsServerTransport
	trEnterpriseMccCustomersServer            *EnterpriseMccCustomersServerTransport
	trIspCacheNodesOperationsServer           *IspCacheNodesOperationsServerTransport
	trIspCustomersServer                      *IspCustomersServerTransport
	trOperationsServer                        *OperationsServerTransport
}

// Do implements the policy.Transporter interface for ServerFactoryTransport.
func (s *ServerFactoryTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	client := method[:strings.Index(method, ".")]
	var resp *http.Response
	var err error

	switch client {
	case "CacheNodesOperationsClient":
		initServer(s, &s.trCacheNodesOperationsServer, func() *CacheNodesOperationsServerTransport {
			return NewCacheNodesOperationsServerTransport(&s.srv.CacheNodesOperationsServer)
		})
		resp, err = s.trCacheNodesOperationsServer.Do(req)
	case "EnterpriseCustomerOperationsClient":
		initServer(s, &s.trEnterpriseCustomerOperationsServer, func() *EnterpriseCustomerOperationsServerTransport {
			return NewEnterpriseCustomerOperationsServerTransport(&s.srv.EnterpriseCustomerOperationsServer)
		})
		resp, err = s.trEnterpriseCustomerOperationsServer.Do(req)
	case "EnterpriseMccCacheNodesOperationsClient":
		initServer(s, &s.trEnterpriseMccCacheNodesOperationsServer, func() *EnterpriseMccCacheNodesOperationsServerTransport {
			return NewEnterpriseMccCacheNodesOperationsServerTransport(&s.srv.EnterpriseMccCacheNodesOperationsServer)
		})
		resp, err = s.trEnterpriseMccCacheNodesOperationsServer.Do(req)
	case "EnterpriseMccCustomersClient":
		initServer(s, &s.trEnterpriseMccCustomersServer, func() *EnterpriseMccCustomersServerTransport {
			return NewEnterpriseMccCustomersServerTransport(&s.srv.EnterpriseMccCustomersServer)
		})
		resp, err = s.trEnterpriseMccCustomersServer.Do(req)
	case "IspCacheNodesOperationsClient":
		initServer(s, &s.trIspCacheNodesOperationsServer, func() *IspCacheNodesOperationsServerTransport {
			return NewIspCacheNodesOperationsServerTransport(&s.srv.IspCacheNodesOperationsServer)
		})
		resp, err = s.trIspCacheNodesOperationsServer.Do(req)
	case "IspCustomersClient":
		initServer(s, &s.trIspCustomersServer, func() *IspCustomersServerTransport { return NewIspCustomersServerTransport(&s.srv.IspCustomersServer) })
		resp, err = s.trIspCustomersServer.Do(req)
	case "OperationsClient":
		initServer(s, &s.trOperationsServer, func() *OperationsServerTransport { return NewOperationsServerTransport(&s.srv.OperationsServer) })
		resp, err = s.trOperationsServer.Do(req)
	default:
		err = fmt.Errorf("unhandled client %s", client)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func initServer[T any](s *ServerFactoryTransport, dst **T, src func() *T) {
	s.trMu.Lock()
	if *dst == nil {
		*dst = src()
	}
	s.trMu.Unlock()
}
