// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) Go Code Generator. DO NOT EDIT.

package armdeviceregistry

import (
	"context"
	"errors"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/arm"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/policy"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"net/url"
	"strings"
)

// BillingContainersClient contains the methods for the BillingContainers group.
// Don't use this type directly, use NewBillingContainersClient() instead.
type BillingContainersClient struct {
	internal       *arm.Client
	subscriptionID string
}

// NewBillingContainersClient creates a new instance of BillingContainersClient with the specified values.
//   - subscriptionID - The ID of the target subscription. The value must be an UUID.
//   - credential - used to authorize requests. Usually a credential from azidentity.
//   - options - pass nil to accept the default values.
func NewBillingContainersClient(subscriptionID string, credential azcore.TokenCredential, options *arm.ClientOptions) (*BillingContainersClient, error) {
	cl, err := arm.NewClient(moduleName, moduleVersion, credential, options)
	if err != nil {
		return nil, err
	}
	client := &BillingContainersClient{
		subscriptionID: subscriptionID,
		internal:       cl,
	}
	return client, nil
}

// Get - Get a BillingContainer
// If the operation fails it returns an *azcore.ResponseError type.
//
// Generated from API version v2024_09_01_preview
//   - billingContainerName - Name of the billing container.
//   - options - BillingContainersClientGetOptions contains the optional parameters for the BillingContainersClient.Get method.
func (client *BillingContainersClient) Get(ctx context.Context, billingContainerName string, options *BillingContainersClientGetOptions) (BillingContainersClientGetResponse, error) {
	var err error
	const operationName = "BillingContainersClient.Get"
	ctx = context.WithValue(ctx, runtime.CtxAPINameKey{}, operationName)
	ctx, endSpan := runtime.StartSpan(ctx, operationName, client.internal.Tracer(), nil)
	defer func() { endSpan(err) }()
	req, err := client.getCreateRequest(ctx, billingContainerName, options)
	if err != nil {
		return BillingContainersClientGetResponse{}, err
	}
	httpResp, err := client.internal.Pipeline().Do(req)
	if err != nil {
		return BillingContainersClientGetResponse{}, err
	}
	if !runtime.HasStatusCode(httpResp, http.StatusOK) {
		err = runtime.NewResponseError(httpResp)
		return BillingContainersClientGetResponse{}, err
	}
	resp, err := client.getHandleResponse(httpResp)
	return resp, err
}

// getCreateRequest creates the Get request.
func (client *BillingContainersClient) getCreateRequest(ctx context.Context, billingContainerName string, _ *BillingContainersClientGetOptions) (*policy.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/providers/Microsoft.DeviceRegistry/billingContainers/{billingContainerName}"
	if client.subscriptionID == "" {
		return nil, errors.New("parameter client.subscriptionID cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	if billingContainerName == "" {
		return nil, errors.New("parameter billingContainerName cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{billingContainerName}", url.PathEscape(billingContainerName))
	req, err := runtime.NewRequest(ctx, http.MethodGet, runtime.JoinPaths(client.internal.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	reqQP := req.Raw().URL.Query()
	reqQP.Set("api-version", "v2024_09_01_preview")
	req.Raw().URL.RawQuery = reqQP.Encode()
	req.Raw().Header["Accept"] = []string{"application/json"}
	return req, nil
}

// getHandleResponse handles the Get response.
func (client *BillingContainersClient) getHandleResponse(resp *http.Response) (BillingContainersClientGetResponse, error) {
	result := BillingContainersClientGetResponse{}
	if err := runtime.UnmarshalAsJSON(resp, &result.BillingContainer); err != nil {
		return BillingContainersClientGetResponse{}, err
	}
	return result, nil
}

// NewListBySubscriptionPager - List BillingContainer resources by subscription ID
//
// Generated from API version v2024_09_01_preview
//   - options - BillingContainersClientListBySubscriptionOptions contains the optional parameters for the BillingContainersClient.NewListBySubscriptionPager
//     method.
func (client *BillingContainersClient) NewListBySubscriptionPager(options *BillingContainersClientListBySubscriptionOptions) *runtime.Pager[BillingContainersClientListBySubscriptionResponse] {
	return runtime.NewPager(runtime.PagingHandler[BillingContainersClientListBySubscriptionResponse]{
		More: func(page BillingContainersClientListBySubscriptionResponse) bool {
			return page.NextLink != nil && len(*page.NextLink) > 0
		},
		Fetcher: func(ctx context.Context, page *BillingContainersClientListBySubscriptionResponse) (BillingContainersClientListBySubscriptionResponse, error) {
			ctx = context.WithValue(ctx, runtime.CtxAPINameKey{}, "BillingContainersClient.NewListBySubscriptionPager")
			nextLink := ""
			if page != nil {
				nextLink = *page.NextLink
			}
			resp, err := runtime.FetcherForNextLink(ctx, client.internal.Pipeline(), nextLink, func(ctx context.Context) (*policy.Request, error) {
				return client.listBySubscriptionCreateRequest(ctx, options)
			}, nil)
			if err != nil {
				return BillingContainersClientListBySubscriptionResponse{}, err
			}
			return client.listBySubscriptionHandleResponse(resp)
		},
		Tracer: client.internal.Tracer(),
	})
}

// listBySubscriptionCreateRequest creates the ListBySubscription request.
func (client *BillingContainersClient) listBySubscriptionCreateRequest(ctx context.Context, _ *BillingContainersClientListBySubscriptionOptions) (*policy.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/providers/Microsoft.DeviceRegistry/billingContainers"
	if client.subscriptionID == "" {
		return nil, errors.New("parameter client.subscriptionID cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	req, err := runtime.NewRequest(ctx, http.MethodGet, runtime.JoinPaths(client.internal.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	reqQP := req.Raw().URL.Query()
	reqQP.Set("api-version", "v2024_09_01_preview")
	req.Raw().URL.RawQuery = reqQP.Encode()
	req.Raw().Header["Accept"] = []string{"application/json"}
	return req, nil
}

// listBySubscriptionHandleResponse handles the ListBySubscription response.
func (client *BillingContainersClient) listBySubscriptionHandleResponse(resp *http.Response) (BillingContainersClientListBySubscriptionResponse, error) {
	result := BillingContainersClientListBySubscriptionResponse{}
	if err := runtime.UnmarshalAsJSON(resp, &result.BillingContainerListResult); err != nil {
		return BillingContainersClientListBySubscriptionResponse{}, err
	}
	return result, nil
}
