// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) Go Code Generator. DO NOT EDIT.

package fake

import (
	"context"
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/edgezones/armedgezones"
	"net/http"
	"net/url"
	"regexp"
)

// ExtendedZonesServer is a fake server for instances of the armedgezones.ExtendedZonesClient type.
type ExtendedZonesServer struct {
	// Get is the fake for method ExtendedZonesClient.Get
	// HTTP status codes to indicate success: http.StatusOK
	Get func(ctx context.Context, extendedZoneName string, options *armedgezones.ExtendedZonesClientGetOptions) (resp azfake.Responder[armedgezones.ExtendedZonesClientGetResponse], errResp azfake.ErrorResponder)

	// NewListBySubscriptionPager is the fake for method ExtendedZonesClient.NewListBySubscriptionPager
	// HTTP status codes to indicate success: http.StatusOK
	NewListBySubscriptionPager func(options *armedgezones.ExtendedZonesClientListBySubscriptionOptions) (resp azfake.PagerResponder[armedgezones.ExtendedZonesClientListBySubscriptionResponse])

	// Register is the fake for method ExtendedZonesClient.Register
	// HTTP status codes to indicate success: http.StatusOK
	Register func(ctx context.Context, extendedZoneName string, options *armedgezones.ExtendedZonesClientRegisterOptions) (resp azfake.Responder[armedgezones.ExtendedZonesClientRegisterResponse], errResp azfake.ErrorResponder)

	// Unregister is the fake for method ExtendedZonesClient.Unregister
	// HTTP status codes to indicate success: http.StatusOK
	Unregister func(ctx context.Context, extendedZoneName string, options *armedgezones.ExtendedZonesClientUnregisterOptions) (resp azfake.Responder[armedgezones.ExtendedZonesClientUnregisterResponse], errResp azfake.ErrorResponder)
}

// NewExtendedZonesServerTransport creates a new instance of ExtendedZonesServerTransport with the provided implementation.
// The returned ExtendedZonesServerTransport instance is connected to an instance of armedgezones.ExtendedZonesClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewExtendedZonesServerTransport(srv *ExtendedZonesServer) *ExtendedZonesServerTransport {
	return &ExtendedZonesServerTransport{
		srv:                        srv,
		newListBySubscriptionPager: newTracker[azfake.PagerResponder[armedgezones.ExtendedZonesClientListBySubscriptionResponse]](),
	}
}

// ExtendedZonesServerTransport connects instances of armedgezones.ExtendedZonesClient to instances of ExtendedZonesServer.
// Don't use this type directly, use NewExtendedZonesServerTransport instead.
type ExtendedZonesServerTransport struct {
	srv                        *ExtendedZonesServer
	newListBySubscriptionPager *tracker[azfake.PagerResponder[armedgezones.ExtendedZonesClientListBySubscriptionResponse]]
}

// Do implements the policy.Transporter interface for ExtendedZonesServerTransport.
func (e *ExtendedZonesServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	return e.dispatchToMethodFake(req, method)
}

func (e *ExtendedZonesServerTransport) dispatchToMethodFake(req *http.Request, method string) (*http.Response, error) {
	var resp *http.Response
	var err error

	switch method {
	case "ExtendedZonesClient.Get":
		resp, err = e.dispatchGet(req)
	case "ExtendedZonesClient.NewListBySubscriptionPager":
		resp, err = e.dispatchNewListBySubscriptionPager(req)
	case "ExtendedZonesClient.Register":
		resp, err = e.dispatchRegister(req)
	case "ExtendedZonesClient.Unregister":
		resp, err = e.dispatchUnregister(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	return resp, err
}

func (e *ExtendedZonesServerTransport) dispatchGet(req *http.Request) (*http.Response, error) {
	if e.srv.Get == nil {
		return nil, &nonRetriableError{errors.New("fake for method Get not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.EdgeZones/extendedZones/(?P<extendedZoneName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 2 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	extendedZoneNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("extendedZoneName")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := e.srv.Get(req.Context(), extendedZoneNameParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).ExtendedZone, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (e *ExtendedZonesServerTransport) dispatchNewListBySubscriptionPager(req *http.Request) (*http.Response, error) {
	if e.srv.NewListBySubscriptionPager == nil {
		return nil, &nonRetriableError{errors.New("fake for method NewListBySubscriptionPager not implemented")}
	}
	newListBySubscriptionPager := e.newListBySubscriptionPager.get(req)
	if newListBySubscriptionPager == nil {
		const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.EdgeZones/extendedZones`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 1 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		resp := e.srv.NewListBySubscriptionPager(nil)
		newListBySubscriptionPager = &resp
		e.newListBySubscriptionPager.add(req, newListBySubscriptionPager)
		server.PagerResponderInjectNextLinks(newListBySubscriptionPager, req, func(page *armedgezones.ExtendedZonesClientListBySubscriptionResponse, createLink func() string) {
			page.NextLink = to.Ptr(createLink())
		})
	}
	resp, err := server.PagerResponderNext(newListBySubscriptionPager, req)
	if err != nil {
		return nil, err
	}
	if !contains([]int{http.StatusOK}, resp.StatusCode) {
		e.newListBySubscriptionPager.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", resp.StatusCode)}
	}
	if !server.PagerResponderMore(newListBySubscriptionPager) {
		e.newListBySubscriptionPager.remove(req)
	}
	return resp, nil
}

func (e *ExtendedZonesServerTransport) dispatchRegister(req *http.Request) (*http.Response, error) {
	if e.srv.Register == nil {
		return nil, &nonRetriableError{errors.New("fake for method Register not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.EdgeZones/extendedZones/(?P<extendedZoneName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/register`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 2 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	extendedZoneNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("extendedZoneName")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := e.srv.Register(req.Context(), extendedZoneNameParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).ExtendedZone, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (e *ExtendedZonesServerTransport) dispatchUnregister(req *http.Request) (*http.Response, error) {
	if e.srv.Unregister == nil {
		return nil, &nonRetriableError{errors.New("fake for method Unregister not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.EdgeZones/extendedZones/(?P<extendedZoneName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/unregister`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 2 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	extendedZoneNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("extendedZoneName")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := e.srv.Unregister(req.Context(), extendedZoneNameParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).ExtendedZone, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}
