//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package armhdinsightcontainers

import (
	"context"
	"errors"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/arm"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/policy"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"net/url"
	"strings"
)

// ClusterLibrariesClient contains the methods for the ClusterLibraries group.
// Don't use this type directly, use NewClusterLibrariesClient() instead.
type ClusterLibrariesClient struct {
	internal       *arm.Client
	subscriptionID string
}

// NewClusterLibrariesClient creates a new instance of ClusterLibrariesClient with the specified values.
//   - subscriptionID - The ID of the target subscription. The value must be an UUID.
//   - credential - used to authorize requests. Usually a credential from azidentity.
//   - options - pass nil to accept the default values.
func NewClusterLibrariesClient(subscriptionID string, credential azcore.TokenCredential, options *arm.ClientOptions) (*ClusterLibrariesClient, error) {
	cl, err := arm.NewClient(moduleName, moduleVersion, credential, options)
	if err != nil {
		return nil, err
	}
	client := &ClusterLibrariesClient{
		subscriptionID: subscriptionID,
		internal:       cl,
	}
	return client, nil
}

// NewListPager - Get all libraries of HDInsight on AKS cluster.
//
// Generated from API version 2024-05-01-preview
//   - resourceGroupName - The name of the resource group. The name is case insensitive.
//   - clusterPoolName - The name of the cluster pool.
//   - clusterName - The name of the HDInsight cluster.
//   - category - The system query option to filter libraries returned in the response. Allowed value is 'custom' or 'predefined'.
//   - options - ClusterLibrariesClientListOptions contains the optional parameters for the ClusterLibrariesClient.NewListPager
//     method.
func (client *ClusterLibrariesClient) NewListPager(resourceGroupName string, clusterPoolName string, clusterName string, category Category, options *ClusterLibrariesClientListOptions) *runtime.Pager[ClusterLibrariesClientListResponse] {
	return runtime.NewPager(runtime.PagingHandler[ClusterLibrariesClientListResponse]{
		More: func(page ClusterLibrariesClientListResponse) bool {
			return page.NextLink != nil && len(*page.NextLink) > 0
		},
		Fetcher: func(ctx context.Context, page *ClusterLibrariesClientListResponse) (ClusterLibrariesClientListResponse, error) {
			ctx = context.WithValue(ctx, runtime.CtxAPINameKey{}, "ClusterLibrariesClient.NewListPager")
			nextLink := ""
			if page != nil {
				nextLink = *page.NextLink
			}
			resp, err := runtime.FetcherForNextLink(ctx, client.internal.Pipeline(), nextLink, func(ctx context.Context) (*policy.Request, error) {
				return client.listCreateRequest(ctx, resourceGroupName, clusterPoolName, clusterName, category, options)
			}, nil)
			if err != nil {
				return ClusterLibrariesClientListResponse{}, err
			}
			return client.listHandleResponse(resp)
		},
		Tracer: client.internal.Tracer(),
	})
}

// listCreateRequest creates the List request.
func (client *ClusterLibrariesClient) listCreateRequest(ctx context.Context, resourceGroupName string, clusterPoolName string, clusterName string, category Category, options *ClusterLibrariesClientListOptions) (*policy.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.HDInsight/clusterpools/{clusterPoolName}/clusters/{clusterName}/libraries"
	if client.subscriptionID == "" {
		return nil, errors.New("parameter client.subscriptionID cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	if resourceGroupName == "" {
		return nil, errors.New("parameter resourceGroupName cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{resourceGroupName}", url.PathEscape(resourceGroupName))
	if clusterPoolName == "" {
		return nil, errors.New("parameter clusterPoolName cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{clusterPoolName}", url.PathEscape(clusterPoolName))
	if clusterName == "" {
		return nil, errors.New("parameter clusterName cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{clusterName}", url.PathEscape(clusterName))
	req, err := runtime.NewRequest(ctx, http.MethodGet, runtime.JoinPaths(client.internal.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	reqQP := req.Raw().URL.Query()
	reqQP.Set("$category", string(category))
	reqQP.Set("api-version", "2024-05-01-preview")
	req.Raw().URL.RawQuery = reqQP.Encode()
	req.Raw().Header["Accept"] = []string{"application/json"}
	return req, nil
}

// listHandleResponse handles the List response.
func (client *ClusterLibrariesClient) listHandleResponse(resp *http.Response) (ClusterLibrariesClientListResponse, error) {
	result := ClusterLibrariesClientListResponse{}
	if err := runtime.UnmarshalAsJSON(resp, &result.ClusterLibraryList); err != nil {
		return ClusterLibrariesClientListResponse{}, err
	}
	return result, nil
}

// BeginManageLibraries - Library management operations on HDInsight on AKS cluster.
// If the operation fails it returns an *azcore.ResponseError type.
//
// Generated from API version 2024-05-01-preview
//   - resourceGroupName - The name of the resource group. The name is case insensitive.
//   - clusterPoolName - The name of the cluster pool.
//   - clusterName - The name of the HDInsight cluster.
//   - operation - The library management operation.
//   - options - ClusterLibrariesClientBeginManageLibrariesOptions contains the optional parameters for the ClusterLibrariesClient.BeginManageLibraries
//     method.
func (client *ClusterLibrariesClient) BeginManageLibraries(ctx context.Context, resourceGroupName string, clusterPoolName string, clusterName string, operation ClusterLibraryManagementOperation, options *ClusterLibrariesClientBeginManageLibrariesOptions) (*runtime.Poller[ClusterLibrariesClientManageLibrariesResponse], error) {
	if options == nil || options.ResumeToken == "" {
		resp, err := client.manageLibraries(ctx, resourceGroupName, clusterPoolName, clusterName, operation, options)
		if err != nil {
			return nil, err
		}
		poller, err := runtime.NewPoller(resp, client.internal.Pipeline(), &runtime.NewPollerOptions[ClusterLibrariesClientManageLibrariesResponse]{
			FinalStateVia: runtime.FinalStateViaLocation,
			Tracer:        client.internal.Tracer(),
		})
		return poller, err
	} else {
		return runtime.NewPollerFromResumeToken(options.ResumeToken, client.internal.Pipeline(), &runtime.NewPollerFromResumeTokenOptions[ClusterLibrariesClientManageLibrariesResponse]{
			Tracer: client.internal.Tracer(),
		})
	}
}

// ManageLibraries - Library management operations on HDInsight on AKS cluster.
// If the operation fails it returns an *azcore.ResponseError type.
//
// Generated from API version 2024-05-01-preview
func (client *ClusterLibrariesClient) manageLibraries(ctx context.Context, resourceGroupName string, clusterPoolName string, clusterName string, operation ClusterLibraryManagementOperation, options *ClusterLibrariesClientBeginManageLibrariesOptions) (*http.Response, error) {
	var err error
	const operationName = "ClusterLibrariesClient.BeginManageLibraries"
	ctx = context.WithValue(ctx, runtime.CtxAPINameKey{}, operationName)
	ctx, endSpan := runtime.StartSpan(ctx, operationName, client.internal.Tracer(), nil)
	defer func() { endSpan(err) }()
	req, err := client.manageLibrariesCreateRequest(ctx, resourceGroupName, clusterPoolName, clusterName, operation, options)
	if err != nil {
		return nil, err
	}
	httpResp, err := client.internal.Pipeline().Do(req)
	if err != nil {
		return nil, err
	}
	if !runtime.HasStatusCode(httpResp, http.StatusAccepted) {
		err = runtime.NewResponseError(httpResp)
		return nil, err
	}
	return httpResp, nil
}

// manageLibrariesCreateRequest creates the ManageLibraries request.
func (client *ClusterLibrariesClient) manageLibrariesCreateRequest(ctx context.Context, resourceGroupName string, clusterPoolName string, clusterName string, operation ClusterLibraryManagementOperation, options *ClusterLibrariesClientBeginManageLibrariesOptions) (*policy.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.HDInsight/clusterpools/{clusterPoolName}/clusters/{clusterName}/manageLibraries"
	if client.subscriptionID == "" {
		return nil, errors.New("parameter client.subscriptionID cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	if resourceGroupName == "" {
		return nil, errors.New("parameter resourceGroupName cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{resourceGroupName}", url.PathEscape(resourceGroupName))
	if clusterPoolName == "" {
		return nil, errors.New("parameter clusterPoolName cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{clusterPoolName}", url.PathEscape(clusterPoolName))
	if clusterName == "" {
		return nil, errors.New("parameter clusterName cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{clusterName}", url.PathEscape(clusterName))
	req, err := runtime.NewRequest(ctx, http.MethodPost, runtime.JoinPaths(client.internal.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	reqQP := req.Raw().URL.Query()
	reqQP.Set("api-version", "2024-05-01-preview")
	req.Raw().URL.RawQuery = reqQP.Encode()
	req.Raw().Header["Accept"] = []string{"application/json"}
	if err := runtime.MarshalAsJSON(req, operation); err != nil {
		return nil, err
	}
	return req, nil
}
