//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"context"
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/oracledatabase/armoracledatabase"
	"net/http"
	"net/url"
	"regexp"
)

// GiVersionsServer is a fake server for instances of the armoracledatabase.GiVersionsClient type.
type GiVersionsServer struct {
	// Get is the fake for method GiVersionsClient.Get
	// HTTP status codes to indicate success: http.StatusOK
	Get func(ctx context.Context, location string, giversionname string, options *armoracledatabase.GiVersionsClientGetOptions) (resp azfake.Responder[armoracledatabase.GiVersionsClientGetResponse], errResp azfake.ErrorResponder)

	// NewListByLocationPager is the fake for method GiVersionsClient.NewListByLocationPager
	// HTTP status codes to indicate success: http.StatusOK
	NewListByLocationPager func(location string, options *armoracledatabase.GiVersionsClientListByLocationOptions) (resp azfake.PagerResponder[armoracledatabase.GiVersionsClientListByLocationResponse])
}

// NewGiVersionsServerTransport creates a new instance of GiVersionsServerTransport with the provided implementation.
// The returned GiVersionsServerTransport instance is connected to an instance of armoracledatabase.GiVersionsClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewGiVersionsServerTransport(srv *GiVersionsServer) *GiVersionsServerTransport {
	return &GiVersionsServerTransport{
		srv:                    srv,
		newListByLocationPager: newTracker[azfake.PagerResponder[armoracledatabase.GiVersionsClientListByLocationResponse]](),
	}
}

// GiVersionsServerTransport connects instances of armoracledatabase.GiVersionsClient to instances of GiVersionsServer.
// Don't use this type directly, use NewGiVersionsServerTransport instead.
type GiVersionsServerTransport struct {
	srv                    *GiVersionsServer
	newListByLocationPager *tracker[azfake.PagerResponder[armoracledatabase.GiVersionsClientListByLocationResponse]]
}

// Do implements the policy.Transporter interface for GiVersionsServerTransport.
func (g *GiVersionsServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "GiVersionsClient.Get":
		resp, err = g.dispatchGet(req)
	case "GiVersionsClient.NewListByLocationPager":
		resp, err = g.dispatchNewListByLocationPager(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (g *GiVersionsServerTransport) dispatchGet(req *http.Request) (*http.Response, error) {
	if g.srv.Get == nil {
		return nil, &nonRetriableError{errors.New("fake for method Get not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Oracle\.Database/locations/(?P<location>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/giVersions/(?P<giversionname>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 3 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	locationParam, err := url.PathUnescape(matches[regex.SubexpIndex("location")])
	if err != nil {
		return nil, err
	}
	giversionnameParam, err := url.PathUnescape(matches[regex.SubexpIndex("giversionname")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := g.srv.Get(req.Context(), locationParam, giversionnameParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).GiVersion, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (g *GiVersionsServerTransport) dispatchNewListByLocationPager(req *http.Request) (*http.Response, error) {
	if g.srv.NewListByLocationPager == nil {
		return nil, &nonRetriableError{errors.New("fake for method NewListByLocationPager not implemented")}
	}
	newListByLocationPager := g.newListByLocationPager.get(req)
	if newListByLocationPager == nil {
		const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Oracle\.Database/locations/(?P<location>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/giVersions`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 2 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		locationParam, err := url.PathUnescape(matches[regex.SubexpIndex("location")])
		if err != nil {
			return nil, err
		}
		resp := g.srv.NewListByLocationPager(locationParam, nil)
		newListByLocationPager = &resp
		g.newListByLocationPager.add(req, newListByLocationPager)
		server.PagerResponderInjectNextLinks(newListByLocationPager, req, func(page *armoracledatabase.GiVersionsClientListByLocationResponse, createLink func() string) {
			page.NextLink = to.Ptr(createLink())
		})
	}
	resp, err := server.PagerResponderNext(newListByLocationPager, req)
	if err != nil {
		return nil, err
	}
	if !contains([]int{http.StatusOK}, resp.StatusCode) {
		g.newListByLocationPager.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", resp.StatusCode)}
	}
	if !server.PagerResponderMore(newListByLocationPager) {
		g.newListByLocationPager.remove(req)
	}
	return resp, nil
}
