//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"context"
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/oracledatabase/armoracledatabase"
	"net/http"
	"net/url"
	"regexp"
)

// SystemVersionsServer is a fake server for instances of the armoracledatabase.SystemVersionsClient type.
type SystemVersionsServer struct {
	// Get is the fake for method SystemVersionsClient.Get
	// HTTP status codes to indicate success: http.StatusOK
	Get func(ctx context.Context, location string, systemversionname string, options *armoracledatabase.SystemVersionsClientGetOptions) (resp azfake.Responder[armoracledatabase.SystemVersionsClientGetResponse], errResp azfake.ErrorResponder)

	// NewListByLocationPager is the fake for method SystemVersionsClient.NewListByLocationPager
	// HTTP status codes to indicate success: http.StatusOK
	NewListByLocationPager func(location string, options *armoracledatabase.SystemVersionsClientListByLocationOptions) (resp azfake.PagerResponder[armoracledatabase.SystemVersionsClientListByLocationResponse])
}

// NewSystemVersionsServerTransport creates a new instance of SystemVersionsServerTransport with the provided implementation.
// The returned SystemVersionsServerTransport instance is connected to an instance of armoracledatabase.SystemVersionsClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewSystemVersionsServerTransport(srv *SystemVersionsServer) *SystemVersionsServerTransport {
	return &SystemVersionsServerTransport{
		srv:                    srv,
		newListByLocationPager: newTracker[azfake.PagerResponder[armoracledatabase.SystemVersionsClientListByLocationResponse]](),
	}
}

// SystemVersionsServerTransport connects instances of armoracledatabase.SystemVersionsClient to instances of SystemVersionsServer.
// Don't use this type directly, use NewSystemVersionsServerTransport instead.
type SystemVersionsServerTransport struct {
	srv                    *SystemVersionsServer
	newListByLocationPager *tracker[azfake.PagerResponder[armoracledatabase.SystemVersionsClientListByLocationResponse]]
}

// Do implements the policy.Transporter interface for SystemVersionsServerTransport.
func (s *SystemVersionsServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "SystemVersionsClient.Get":
		resp, err = s.dispatchGet(req)
	case "SystemVersionsClient.NewListByLocationPager":
		resp, err = s.dispatchNewListByLocationPager(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (s *SystemVersionsServerTransport) dispatchGet(req *http.Request) (*http.Response, error) {
	if s.srv.Get == nil {
		return nil, &nonRetriableError{errors.New("fake for method Get not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Oracle\.Database/locations/(?P<location>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/systemVersions/(?P<systemversionname>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 3 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	locationParam, err := url.PathUnescape(matches[regex.SubexpIndex("location")])
	if err != nil {
		return nil, err
	}
	systemversionnameParam, err := url.PathUnescape(matches[regex.SubexpIndex("systemversionname")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := s.srv.Get(req.Context(), locationParam, systemversionnameParam, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).SystemVersion, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (s *SystemVersionsServerTransport) dispatchNewListByLocationPager(req *http.Request) (*http.Response, error) {
	if s.srv.NewListByLocationPager == nil {
		return nil, &nonRetriableError{errors.New("fake for method NewListByLocationPager not implemented")}
	}
	newListByLocationPager := s.newListByLocationPager.get(req)
	if newListByLocationPager == nil {
		const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Oracle\.Database/locations/(?P<location>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/systemVersions`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 2 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		locationParam, err := url.PathUnescape(matches[regex.SubexpIndex("location")])
		if err != nil {
			return nil, err
		}
		resp := s.srv.NewListByLocationPager(locationParam, nil)
		newListByLocationPager = &resp
		s.newListByLocationPager.add(req, newListByLocationPager)
		server.PagerResponderInjectNextLinks(newListByLocationPager, req, func(page *armoracledatabase.SystemVersionsClientListByLocationResponse, createLink func() string) {
			page.NextLink = to.Ptr(createLink())
		})
	}
	resp, err := server.PagerResponderNext(newListByLocationPager, req)
	if err != nil {
		return nil, err
	}
	if !contains([]int{http.StatusOK}, resp.StatusCode) {
		s.newListByLocationPager.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", resp.StatusCode)}
	}
	if !server.PagerResponderMore(newListByLocationPager) {
		s.newListByLocationPager.remove(req)
	}
	return resp, nil
}
