//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/to"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/peering/armpeering"
	"net/http"
	"regexp"
)

// ServiceProvidersServer is a fake server for instances of the armpeering.ServiceProvidersClient type.
type ServiceProvidersServer struct {
	// NewListPager is the fake for method ServiceProvidersClient.NewListPager
	// HTTP status codes to indicate success: http.StatusOK
	NewListPager func(options *armpeering.ServiceProvidersClientListOptions) (resp azfake.PagerResponder[armpeering.ServiceProvidersClientListResponse])
}

// NewServiceProvidersServerTransport creates a new instance of ServiceProvidersServerTransport with the provided implementation.
// The returned ServiceProvidersServerTransport instance is connected to an instance of armpeering.ServiceProvidersClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewServiceProvidersServerTransport(srv *ServiceProvidersServer) *ServiceProvidersServerTransport {
	return &ServiceProvidersServerTransport{
		srv:          srv,
		newListPager: newTracker[azfake.PagerResponder[armpeering.ServiceProvidersClientListResponse]](),
	}
}

// ServiceProvidersServerTransport connects instances of armpeering.ServiceProvidersClient to instances of ServiceProvidersServer.
// Don't use this type directly, use NewServiceProvidersServerTransport instead.
type ServiceProvidersServerTransport struct {
	srv          *ServiceProvidersServer
	newListPager *tracker[azfake.PagerResponder[armpeering.ServiceProvidersClientListResponse]]
}

// Do implements the policy.Transporter interface for ServiceProvidersServerTransport.
func (s *ServiceProvidersServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "ServiceProvidersClient.NewListPager":
		resp, err = s.dispatchNewListPager(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (s *ServiceProvidersServerTransport) dispatchNewListPager(req *http.Request) (*http.Response, error) {
	if s.srv.NewListPager == nil {
		return nil, &nonRetriableError{errors.New("fake for method NewListPager not implemented")}
	}
	newListPager := s.newListPager.get(req)
	if newListPager == nil {
		const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.Peering/peeringServiceProviders`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 1 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		resp := s.srv.NewListPager(nil)
		newListPager = &resp
		s.newListPager.add(req, newListPager)
		server.PagerResponderInjectNextLinks(newListPager, req, func(page *armpeering.ServiceProvidersClientListResponse, createLink func() string) {
			page.NextLink = to.Ptr(createLink())
		})
	}
	resp, err := server.PagerResponderNext(newListPager, req)
	if err != nil {
		return nil, err
	}
	if !contains([]int{http.StatusOK}, resp.StatusCode) {
		s.newListPager.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", resp.StatusCode)}
	}
	if !server.PagerResponderMore(newListPager) {
		s.newListPager.remove(req)
	}
	return resp, nil
}
