//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package armpostgresqlflexibleservers

import (
	"context"
	"errors"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/arm"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/policy"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"net/url"
	"strings"
)

// PostgreSQLServerManagementClient contains the methods for the PostgreSQLServerManagementClient group.
// Don't use this type directly, use NewPostgreSQLServerManagementClient() instead.
type PostgreSQLServerManagementClient struct {
	internal *arm.Client
}

// NewPostgreSQLServerManagementClient creates a new instance of PostgreSQLServerManagementClient with the specified values.
//   - credential - used to authorize requests. Usually a credential from azidentity.
//   - options - pass nil to accept the default values.
func NewPostgreSQLServerManagementClient(credential azcore.TokenCredential, options *arm.ClientOptions) (*PostgreSQLServerManagementClient, error) {
	cl, err := arm.NewClient(moduleName, moduleVersion, credential, options)
	if err != nil {
		return nil, err
	}
	client := &PostgreSQLServerManagementClient{
		internal: cl,
	}
	return client, nil
}

// CheckMigrationNameAvailability - This method checks whether a proposed migration name is valid and available.
// If the operation fails it returns an *azcore.ResponseError type.
//
// Generated from API version 2024-08-01
//   - subscriptionID - The subscription ID of the target database server.
//   - resourceGroupName - The resource group name of the target database server.
//   - targetDbServerName - The name of the target database server.
//   - parameters - The required parameters for checking if a migration name is available.
//   - options - PostgreSQLServerManagementClientCheckMigrationNameAvailabilityOptions contains the optional parameters for the
//     PostgreSQLServerManagementClient.CheckMigrationNameAvailability method.
func (client *PostgreSQLServerManagementClient) CheckMigrationNameAvailability(ctx context.Context, subscriptionID string, resourceGroupName string, targetDbServerName string, parameters MigrationNameAvailabilityResource, options *PostgreSQLServerManagementClientCheckMigrationNameAvailabilityOptions) (PostgreSQLServerManagementClientCheckMigrationNameAvailabilityResponse, error) {
	var err error
	const operationName = "PostgreSQLServerManagementClient.CheckMigrationNameAvailability"
	ctx = context.WithValue(ctx, runtime.CtxAPINameKey{}, operationName)
	ctx, endSpan := runtime.StartSpan(ctx, operationName, client.internal.Tracer(), nil)
	defer func() { endSpan(err) }()
	req, err := client.checkMigrationNameAvailabilityCreateRequest(ctx, subscriptionID, resourceGroupName, targetDbServerName, parameters, options)
	if err != nil {
		return PostgreSQLServerManagementClientCheckMigrationNameAvailabilityResponse{}, err
	}
	httpResp, err := client.internal.Pipeline().Do(req)
	if err != nil {
		return PostgreSQLServerManagementClientCheckMigrationNameAvailabilityResponse{}, err
	}
	if !runtime.HasStatusCode(httpResp, http.StatusOK) {
		err = runtime.NewResponseError(httpResp)
		return PostgreSQLServerManagementClientCheckMigrationNameAvailabilityResponse{}, err
	}
	resp, err := client.checkMigrationNameAvailabilityHandleResponse(httpResp)
	return resp, err
}

// checkMigrationNameAvailabilityCreateRequest creates the CheckMigrationNameAvailability request.
func (client *PostgreSQLServerManagementClient) checkMigrationNameAvailabilityCreateRequest(ctx context.Context, subscriptionID string, resourceGroupName string, targetDbServerName string, parameters MigrationNameAvailabilityResource, options *PostgreSQLServerManagementClientCheckMigrationNameAvailabilityOptions) (*policy.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DBforPostgreSQL/flexibleServers/{targetDbServerName}/checkMigrationNameAvailability"
	if subscriptionID == "" {
		return nil, errors.New("parameter subscriptionID cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(subscriptionID))
	if resourceGroupName == "" {
		return nil, errors.New("parameter resourceGroupName cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{resourceGroupName}", url.PathEscape(resourceGroupName))
	if targetDbServerName == "" {
		return nil, errors.New("parameter targetDbServerName cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{targetDbServerName}", url.PathEscape(targetDbServerName))
	req, err := runtime.NewRequest(ctx, http.MethodPost, runtime.JoinPaths(client.internal.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	reqQP := req.Raw().URL.Query()
	reqQP.Set("api-version", "2024-08-01")
	req.Raw().URL.RawQuery = reqQP.Encode()
	req.Raw().Header["Accept"] = []string{"application/json"}
	if err := runtime.MarshalAsJSON(req, parameters); err != nil {
		return nil, err
	}
	return req, nil
}

// checkMigrationNameAvailabilityHandleResponse handles the CheckMigrationNameAvailability response.
func (client *PostgreSQLServerManagementClient) checkMigrationNameAvailabilityHandleResponse(resp *http.Response) (PostgreSQLServerManagementClientCheckMigrationNameAvailabilityResponse, error) {
	result := PostgreSQLServerManagementClientCheckMigrationNameAvailabilityResponse{}
	if err := runtime.UnmarshalAsJSON(resp, &result.MigrationNameAvailabilityResource); err != nil {
		return PostgreSQLServerManagementClientCheckMigrationNameAvailabilityResponse{}, err
	}
	return result, nil
}
