//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"context"
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/securityinsights/armsecurityinsights/v2"
	"net/http"
	"net/url"
	"regexp"
)

// BookmarkServer is a fake server for instances of the armsecurityinsights.BookmarkClient type.
type BookmarkServer struct {
	// Expand is the fake for method BookmarkClient.Expand
	// HTTP status codes to indicate success: http.StatusOK
	Expand func(ctx context.Context, resourceGroupName string, workspaceName string, bookmarkID string, parameters armsecurityinsights.BookmarkExpandParameters, options *armsecurityinsights.BookmarkClientExpandOptions) (resp azfake.Responder[armsecurityinsights.BookmarkClientExpandResponse], errResp azfake.ErrorResponder)
}

// NewBookmarkServerTransport creates a new instance of BookmarkServerTransport with the provided implementation.
// The returned BookmarkServerTransport instance is connected to an instance of armsecurityinsights.BookmarkClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewBookmarkServerTransport(srv *BookmarkServer) *BookmarkServerTransport {
	return &BookmarkServerTransport{srv: srv}
}

// BookmarkServerTransport connects instances of armsecurityinsights.BookmarkClient to instances of BookmarkServer.
// Don't use this type directly, use NewBookmarkServerTransport instead.
type BookmarkServerTransport struct {
	srv *BookmarkServer
}

// Do implements the policy.Transporter interface for BookmarkServerTransport.
func (b *BookmarkServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "BookmarkClient.Expand":
		resp, err = b.dispatchExpand(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (b *BookmarkServerTransport) dispatchExpand(req *http.Request) (*http.Response, error) {
	if b.srv.Expand == nil {
		return nil, &nonRetriableError{errors.New("fake for method Expand not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/resourceGroups/(?P<resourceGroupName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.OperationalInsights/workspaces/(?P<workspaceName>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.SecurityInsights/bookmarks/(?P<bookmarkId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/expand`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 4 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	body, err := server.UnmarshalRequestAsJSON[armsecurityinsights.BookmarkExpandParameters](req)
	if err != nil {
		return nil, err
	}
	resourceGroupNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("resourceGroupName")])
	if err != nil {
		return nil, err
	}
	workspaceNameParam, err := url.PathUnescape(matches[regex.SubexpIndex("workspaceName")])
	if err != nil {
		return nil, err
	}
	bookmarkIDParam, err := url.PathUnescape(matches[regex.SubexpIndex("bookmarkId")])
	if err != nil {
		return nil, err
	}
	respr, errRespr := b.srv.Expand(req.Context(), resourceGroupNameParam, workspaceNameParam, bookmarkIDParam, body, nil)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).BookmarkExpandResponse, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}
