//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"context"
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/selfhelp/armselfhelp/v2"
	"net/http"
	"net/url"
	"reflect"
	"regexp"
)

// DiscoverySolutionNLPServer is a fake server for instances of the armselfhelp.DiscoverySolutionNLPClient type.
type DiscoverySolutionNLPServer struct {
	// DiscoverSolutions is the fake for method DiscoverySolutionNLPClient.DiscoverSolutions
	// HTTP status codes to indicate success: http.StatusOK
	DiscoverSolutions func(ctx context.Context, options *armselfhelp.DiscoverySolutionNLPClientDiscoverSolutionsOptions) (resp azfake.Responder[armselfhelp.DiscoverySolutionNLPClientDiscoverSolutionsResponse], errResp azfake.ErrorResponder)

	// DiscoverSolutionsBySubscription is the fake for method DiscoverySolutionNLPClient.DiscoverSolutionsBySubscription
	// HTTP status codes to indicate success: http.StatusOK
	DiscoverSolutionsBySubscription func(ctx context.Context, subscriptionID string, options *armselfhelp.DiscoverySolutionNLPClientDiscoverSolutionsBySubscriptionOptions) (resp azfake.Responder[armselfhelp.DiscoverySolutionNLPClientDiscoverSolutionsBySubscriptionResponse], errResp azfake.ErrorResponder)
}

// NewDiscoverySolutionNLPServerTransport creates a new instance of DiscoverySolutionNLPServerTransport with the provided implementation.
// The returned DiscoverySolutionNLPServerTransport instance is connected to an instance of armselfhelp.DiscoverySolutionNLPClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewDiscoverySolutionNLPServerTransport(srv *DiscoverySolutionNLPServer) *DiscoverySolutionNLPServerTransport {
	return &DiscoverySolutionNLPServerTransport{srv: srv}
}

// DiscoverySolutionNLPServerTransport connects instances of armselfhelp.DiscoverySolutionNLPClient to instances of DiscoverySolutionNLPServer.
// Don't use this type directly, use NewDiscoverySolutionNLPServerTransport instead.
type DiscoverySolutionNLPServerTransport struct {
	srv *DiscoverySolutionNLPServer
}

// Do implements the policy.Transporter interface for DiscoverySolutionNLPServerTransport.
func (d *DiscoverySolutionNLPServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	var resp *http.Response
	var err error

	switch method {
	case "DiscoverySolutionNLPClient.DiscoverSolutions":
		resp, err = d.dispatchDiscoverSolutions(req)
	case "DiscoverySolutionNLPClient.DiscoverSolutionsBySubscription":
		resp, err = d.dispatchDiscoverSolutionsBySubscription(req)
	default:
		err = fmt.Errorf("unhandled API %s", method)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func (d *DiscoverySolutionNLPServerTransport) dispatchDiscoverSolutions(req *http.Request) (*http.Response, error) {
	if d.srv.DiscoverSolutions == nil {
		return nil, &nonRetriableError{errors.New("fake for method DiscoverSolutions not implemented")}
	}
	body, err := server.UnmarshalRequestAsJSON[armselfhelp.DiscoveryNlpRequest](req)
	if err != nil {
		return nil, err
	}
	var options *armselfhelp.DiscoverySolutionNLPClientDiscoverSolutionsOptions
	if !reflect.ValueOf(body).IsZero() {
		options = &armselfhelp.DiscoverySolutionNLPClientDiscoverSolutionsOptions{
			DiscoverSolutionRequest: &body,
		}
	}
	respr, errRespr := d.srv.DiscoverSolutions(req.Context(), options)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).DiscoveryNlpResponse, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func (d *DiscoverySolutionNLPServerTransport) dispatchDiscoverSolutionsBySubscription(req *http.Request) (*http.Response, error) {
	if d.srv.DiscoverSolutionsBySubscription == nil {
		return nil, &nonRetriableError{errors.New("fake for method DiscoverSolutionsBySubscription not implemented")}
	}
	const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.Help/discoverSolutions`
	regex := regexp.MustCompile(regexStr)
	matches := regex.FindStringSubmatch(req.URL.EscapedPath())
	if matches == nil || len(matches) < 1 {
		return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
	}
	body, err := server.UnmarshalRequestAsJSON[armselfhelp.DiscoveryNlpRequest](req)
	if err != nil {
		return nil, err
	}
	subscriptionIDParam, err := url.PathUnescape(matches[regex.SubexpIndex("subscriptionId")])
	if err != nil {
		return nil, err
	}
	var options *armselfhelp.DiscoverySolutionNLPClientDiscoverSolutionsBySubscriptionOptions
	if !reflect.ValueOf(body).IsZero() {
		options = &armselfhelp.DiscoverySolutionNLPClientDiscoverSolutionsBySubscriptionOptions{
			DiscoverSolutionRequest: &body,
		}
	}
	respr, errRespr := d.srv.DiscoverSolutionsBySubscription(req.Context(), subscriptionIDParam, options)
	if respErr := server.GetError(errRespr, req); respErr != nil {
		return nil, respErr
	}
	respContent := server.GetResponseContent(respr)
	if !contains([]int{http.StatusOK}, respContent.HTTPStatus) {
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK", respContent.HTTPStatus)}
	}
	resp, err := server.MarshalResponseAsJSON(respContent, server.GetResponse(respr).DiscoveryNlpResponse, req)
	if err != nil {
		return nil, err
	}
	return resp, nil
}
