//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"errors"
	"fmt"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"strings"
	"sync"
)

// ServerFactory is a fake server for instances of the armservicefabric.ClientFactory type.
type ServerFactory struct {
	ApplicationTypeVersionsServer ApplicationTypeVersionsServer
	ApplicationTypesServer        ApplicationTypesServer
	ApplicationsServer            ApplicationsServer
	ClusterVersionsServer         ClusterVersionsServer
	ClustersServer                ClustersServer
	OperationsServer              OperationsServer
	ServicesServer                ServicesServer
}

// NewServerFactoryTransport creates a new instance of ServerFactoryTransport with the provided implementation.
// The returned ServerFactoryTransport instance is connected to an instance of armservicefabric.ClientFactory via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewServerFactoryTransport(srv *ServerFactory) *ServerFactoryTransport {
	return &ServerFactoryTransport{
		srv: srv,
	}
}

// ServerFactoryTransport connects instances of armservicefabric.ClientFactory to instances of ServerFactory.
// Don't use this type directly, use NewServerFactoryTransport instead.
type ServerFactoryTransport struct {
	srv                             *ServerFactory
	trMu                            sync.Mutex
	trApplicationTypeVersionsServer *ApplicationTypeVersionsServerTransport
	trApplicationTypesServer        *ApplicationTypesServerTransport
	trApplicationsServer            *ApplicationsServerTransport
	trClusterVersionsServer         *ClusterVersionsServerTransport
	trClustersServer                *ClustersServerTransport
	trOperationsServer              *OperationsServerTransport
	trServicesServer                *ServicesServerTransport
}

// Do implements the policy.Transporter interface for ServerFactoryTransport.
func (s *ServerFactoryTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	client := method[:strings.Index(method, ".")]
	var resp *http.Response
	var err error

	switch client {
	case "ApplicationTypeVersionsClient":
		initServer(s, &s.trApplicationTypeVersionsServer, func() *ApplicationTypeVersionsServerTransport {
			return NewApplicationTypeVersionsServerTransport(&s.srv.ApplicationTypeVersionsServer)
		})
		resp, err = s.trApplicationTypeVersionsServer.Do(req)
	case "ApplicationTypesClient":
		initServer(s, &s.trApplicationTypesServer, func() *ApplicationTypesServerTransport {
			return NewApplicationTypesServerTransport(&s.srv.ApplicationTypesServer)
		})
		resp, err = s.trApplicationTypesServer.Do(req)
	case "ApplicationsClient":
		initServer(s, &s.trApplicationsServer, func() *ApplicationsServerTransport { return NewApplicationsServerTransport(&s.srv.ApplicationsServer) })
		resp, err = s.trApplicationsServer.Do(req)
	case "ClusterVersionsClient":
		initServer(s, &s.trClusterVersionsServer, func() *ClusterVersionsServerTransport {
			return NewClusterVersionsServerTransport(&s.srv.ClusterVersionsServer)
		})
		resp, err = s.trClusterVersionsServer.Do(req)
	case "ClustersClient":
		initServer(s, &s.trClustersServer, func() *ClustersServerTransport { return NewClustersServerTransport(&s.srv.ClustersServer) })
		resp, err = s.trClustersServer.Do(req)
	case "OperationsClient":
		initServer(s, &s.trOperationsServer, func() *OperationsServerTransport { return NewOperationsServerTransport(&s.srv.OperationsServer) })
		resp, err = s.trOperationsServer.Do(req)
	case "ServicesClient":
		initServer(s, &s.trServicesServer, func() *ServicesServerTransport { return NewServicesServerTransport(&s.srv.ServicesServer) })
		resp, err = s.trServicesServer.Do(req)
	default:
		err = fmt.Errorf("unhandled client %s", client)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func initServer[T any](s *ServerFactoryTransport, dst **T, src func() *T) {
	s.trMu.Lock()
	if *dst == nil {
		*dst = src()
	}
	s.trMu.Unlock()
}
