//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package armservicefabricmanagedclusters

import (
	"context"
	"errors"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/arm"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/policy"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"net/url"
	"strings"
)

// ManagedApplyMaintenanceWindowClient contains the methods for the ManagedApplyMaintenanceWindow group.
// Don't use this type directly, use NewManagedApplyMaintenanceWindowClient() instead.
type ManagedApplyMaintenanceWindowClient struct {
	internal       *arm.Client
	subscriptionID string
}

// NewManagedApplyMaintenanceWindowClient creates a new instance of ManagedApplyMaintenanceWindowClient with the specified values.
//   - subscriptionID - The customer subscription identifier.
//   - credential - used to authorize requests. Usually a credential from azidentity.
//   - options - pass nil to accept the default values.
func NewManagedApplyMaintenanceWindowClient(subscriptionID string, credential azcore.TokenCredential, options *arm.ClientOptions) (*ManagedApplyMaintenanceWindowClient, error) {
	cl, err := arm.NewClient(moduleName, moduleVersion, credential, options)
	if err != nil {
		return nil, err
	}
	client := &ManagedApplyMaintenanceWindowClient{
		subscriptionID: subscriptionID,
		internal:       cl,
	}
	return client, nil
}

// Post - Action to Apply Maintenance window on the Service Fabric Managed Clusters, right now. Any pending update will be
// applied.
// If the operation fails it returns an *azcore.ResponseError type.
//
// Generated from API version 2024-09-01-preview
//   - resourceGroupName - The name of the resource group.
//   - clusterName - The name of the cluster resource.
//   - options - ManagedApplyMaintenanceWindowClientPostOptions contains the optional parameters for the ManagedApplyMaintenanceWindowClient.Post
//     method.
func (client *ManagedApplyMaintenanceWindowClient) Post(ctx context.Context, resourceGroupName string, clusterName string, options *ManagedApplyMaintenanceWindowClientPostOptions) (ManagedApplyMaintenanceWindowClientPostResponse, error) {
	var err error
	const operationName = "ManagedApplyMaintenanceWindowClient.Post"
	ctx = context.WithValue(ctx, runtime.CtxAPINameKey{}, operationName)
	ctx, endSpan := runtime.StartSpan(ctx, operationName, client.internal.Tracer(), nil)
	defer func() { endSpan(err) }()
	req, err := client.postCreateRequest(ctx, resourceGroupName, clusterName, options)
	if err != nil {
		return ManagedApplyMaintenanceWindowClientPostResponse{}, err
	}
	httpResp, err := client.internal.Pipeline().Do(req)
	if err != nil {
		return ManagedApplyMaintenanceWindowClientPostResponse{}, err
	}
	if !runtime.HasStatusCode(httpResp, http.StatusOK) {
		err = runtime.NewResponseError(httpResp)
		return ManagedApplyMaintenanceWindowClientPostResponse{}, err
	}
	return ManagedApplyMaintenanceWindowClientPostResponse{}, nil
}

// postCreateRequest creates the Post request.
func (client *ManagedApplyMaintenanceWindowClient) postCreateRequest(ctx context.Context, resourceGroupName string, clusterName string, options *ManagedApplyMaintenanceWindowClientPostOptions) (*policy.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ServiceFabric/managedClusters/{clusterName}/applyMaintenanceWindow"
	if resourceGroupName == "" {
		return nil, errors.New("parameter resourceGroupName cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{resourceGroupName}", url.PathEscape(resourceGroupName))
	if clusterName == "" {
		return nil, errors.New("parameter clusterName cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{clusterName}", url.PathEscape(clusterName))
	if client.subscriptionID == "" {
		return nil, errors.New("parameter client.subscriptionID cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	req, err := runtime.NewRequest(ctx, http.MethodPost, runtime.JoinPaths(client.internal.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	reqQP := req.Raw().URL.Query()
	reqQP.Set("api-version", "2024-09-01-preview")
	req.Raw().URL.RawQuery = reqQP.Encode()
	req.Raw().Header["Accept"] = []string{"application/json"}
	return req, nil
}
