// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) Go Code Generator. DO NOT EDIT.

package fake

import (
	"errors"
	"fmt"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"strings"
	"sync"
)

// ServerFactory is a fake server for instances of the armstandbypool.ClientFactory type.
type ServerFactory struct {
	// OperationsServer contains the fakes for client OperationsClient
	OperationsServer OperationsServer

	// StandbyContainerGroupPoolRuntimeViewsServer contains the fakes for client StandbyContainerGroupPoolRuntimeViewsClient
	StandbyContainerGroupPoolRuntimeViewsServer StandbyContainerGroupPoolRuntimeViewsServer

	// StandbyContainerGroupPoolsServer contains the fakes for client StandbyContainerGroupPoolsClient
	StandbyContainerGroupPoolsServer StandbyContainerGroupPoolsServer

	// StandbyVirtualMachinePoolRuntimeViewsServer contains the fakes for client StandbyVirtualMachinePoolRuntimeViewsClient
	StandbyVirtualMachinePoolRuntimeViewsServer StandbyVirtualMachinePoolRuntimeViewsServer

	// StandbyVirtualMachinePoolsServer contains the fakes for client StandbyVirtualMachinePoolsClient
	StandbyVirtualMachinePoolsServer StandbyVirtualMachinePoolsServer

	// StandbyVirtualMachinesServer contains the fakes for client StandbyVirtualMachinesClient
	StandbyVirtualMachinesServer StandbyVirtualMachinesServer
}

// NewServerFactoryTransport creates a new instance of ServerFactoryTransport with the provided implementation.
// The returned ServerFactoryTransport instance is connected to an instance of armstandbypool.ClientFactory via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewServerFactoryTransport(srv *ServerFactory) *ServerFactoryTransport {
	return &ServerFactoryTransport{
		srv: srv,
	}
}

// ServerFactoryTransport connects instances of armstandbypool.ClientFactory to instances of ServerFactory.
// Don't use this type directly, use NewServerFactoryTransport instead.
type ServerFactoryTransport struct {
	srv                                           *ServerFactory
	trMu                                          sync.Mutex
	trOperationsServer                            *OperationsServerTransport
	trStandbyContainerGroupPoolRuntimeViewsServer *StandbyContainerGroupPoolRuntimeViewsServerTransport
	trStandbyContainerGroupPoolsServer            *StandbyContainerGroupPoolsServerTransport
	trStandbyVirtualMachinePoolRuntimeViewsServer *StandbyVirtualMachinePoolRuntimeViewsServerTransport
	trStandbyVirtualMachinePoolsServer            *StandbyVirtualMachinePoolsServerTransport
	trStandbyVirtualMachinesServer                *StandbyVirtualMachinesServerTransport
}

// Do implements the policy.Transporter interface for ServerFactoryTransport.
func (s *ServerFactoryTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	client := method[:strings.Index(method, ".")]
	var resp *http.Response
	var err error

	switch client {
	case "OperationsClient":
		initServer(s, &s.trOperationsServer, func() *OperationsServerTransport { return NewOperationsServerTransport(&s.srv.OperationsServer) })
		resp, err = s.trOperationsServer.Do(req)
	case "StandbyContainerGroupPoolRuntimeViewsClient":
		initServer(s, &s.trStandbyContainerGroupPoolRuntimeViewsServer, func() *StandbyContainerGroupPoolRuntimeViewsServerTransport {
			return NewStandbyContainerGroupPoolRuntimeViewsServerTransport(&s.srv.StandbyContainerGroupPoolRuntimeViewsServer)
		})
		resp, err = s.trStandbyContainerGroupPoolRuntimeViewsServer.Do(req)
	case "StandbyContainerGroupPoolsClient":
		initServer(s, &s.trStandbyContainerGroupPoolsServer, func() *StandbyContainerGroupPoolsServerTransport {
			return NewStandbyContainerGroupPoolsServerTransport(&s.srv.StandbyContainerGroupPoolsServer)
		})
		resp, err = s.trStandbyContainerGroupPoolsServer.Do(req)
	case "StandbyVirtualMachinePoolRuntimeViewsClient":
		initServer(s, &s.trStandbyVirtualMachinePoolRuntimeViewsServer, func() *StandbyVirtualMachinePoolRuntimeViewsServerTransport {
			return NewStandbyVirtualMachinePoolRuntimeViewsServerTransport(&s.srv.StandbyVirtualMachinePoolRuntimeViewsServer)
		})
		resp, err = s.trStandbyVirtualMachinePoolRuntimeViewsServer.Do(req)
	case "StandbyVirtualMachinePoolsClient":
		initServer(s, &s.trStandbyVirtualMachinePoolsServer, func() *StandbyVirtualMachinePoolsServerTransport {
			return NewStandbyVirtualMachinePoolsServerTransport(&s.srv.StandbyVirtualMachinePoolsServer)
		})
		resp, err = s.trStandbyVirtualMachinePoolsServer.Do(req)
	case "StandbyVirtualMachinesClient":
		initServer(s, &s.trStandbyVirtualMachinesServer, func() *StandbyVirtualMachinesServerTransport {
			return NewStandbyVirtualMachinesServerTransport(&s.srv.StandbyVirtualMachinesServer)
		})
		resp, err = s.trStandbyVirtualMachinesServer.Do(req)
	default:
		err = fmt.Errorf("unhandled client %s", client)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func initServer[T any](s *ServerFactoryTransport, dst **T, src func() *T) {
	s.trMu.Lock()
	if *dst == nil {
		*dst = src()
	}
	s.trMu.Unlock()
}
