// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) Go Code Generator. DO NOT EDIT.

package fake

import (
	"context"
	"errors"
	"fmt"
	azfake "github.com/Azure/azure-sdk-for-go/sdk/azcore/fake"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/fake/server"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/terraform/armterraform"
	"net/http"
	"regexp"
)

// TerraformServer is a fake server for instances of the armterraform.TerraformClient type.
type TerraformServer struct {
	// BeginExportTerraform is the fake for method TerraformClient.BeginExportTerraform
	// HTTP status codes to indicate success: http.StatusOK, http.StatusAccepted
	BeginExportTerraform func(ctx context.Context, body armterraform.BaseExportModelClassification, options *armterraform.TerraformClientBeginExportTerraformOptions) (resp azfake.PollerResponder[armterraform.TerraformClientExportTerraformResponse], errResp azfake.ErrorResponder)
}

// NewTerraformServerTransport creates a new instance of TerraformServerTransport with the provided implementation.
// The returned TerraformServerTransport instance is connected to an instance of armterraform.TerraformClient via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewTerraformServerTransport(srv *TerraformServer) *TerraformServerTransport {
	return &TerraformServerTransport{
		srv:                  srv,
		beginExportTerraform: newTracker[azfake.PollerResponder[armterraform.TerraformClientExportTerraformResponse]](),
	}
}

// TerraformServerTransport connects instances of armterraform.TerraformClient to instances of TerraformServer.
// Don't use this type directly, use NewTerraformServerTransport instead.
type TerraformServerTransport struct {
	srv                  *TerraformServer
	beginExportTerraform *tracker[azfake.PollerResponder[armterraform.TerraformClientExportTerraformResponse]]
}

// Do implements the policy.Transporter interface for TerraformServerTransport.
func (t *TerraformServerTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	return t.dispatchToMethodFake(req, method)
}

func (t *TerraformServerTransport) dispatchToMethodFake(req *http.Request, method string) (*http.Response, error) {
	resultChan := make(chan result)
	defer close(resultChan)

	go func() {
		var intercepted bool
		var res result
		if terraformServerTransportInterceptor != nil {
			res.resp, res.err, intercepted = terraformServerTransportInterceptor.Do(req)
		}
		if !intercepted {
			switch method {
			case "TerraformClient.BeginExportTerraform":
				res.resp, res.err = t.dispatchBeginExportTerraform(req)
			default:
				res.err = fmt.Errorf("unhandled API %s", method)
			}

		}
		select {
		case resultChan <- res:
		case <-req.Context().Done():
		}
	}()

	select {
	case <-req.Context().Done():
		return nil, req.Context().Err()
	case res := <-resultChan:
		return res.resp, res.err
	}
}

func (t *TerraformServerTransport) dispatchBeginExportTerraform(req *http.Request) (*http.Response, error) {
	if t.srv.BeginExportTerraform == nil {
		return nil, &nonRetriableError{errors.New("fake for method BeginExportTerraform not implemented")}
	}
	beginExportTerraform := t.beginExportTerraform.get(req)
	if beginExportTerraform == nil {
		const regexStr = `/subscriptions/(?P<subscriptionId>[!#&$-;=?-\[\]_a-zA-Z0-9~%@]+)/providers/Microsoft\.AzureTerraform/exportTerraform`
		regex := regexp.MustCompile(regexStr)
		matches := regex.FindStringSubmatch(req.URL.EscapedPath())
		if matches == nil || len(matches) < 1 {
			return nil, fmt.Errorf("failed to parse path %s", req.URL.Path)
		}
		raw, err := readRequestBody(req)
		if err != nil {
			return nil, err
		}
		body, err := unmarshalBaseExportModelClassification(raw)
		if err != nil {
			return nil, err
		}
		respr, errRespr := t.srv.BeginExportTerraform(req.Context(), body, nil)
		if respErr := server.GetError(errRespr, req); respErr != nil {
			return nil, respErr
		}
		beginExportTerraform = &respr
		t.beginExportTerraform.add(req, beginExportTerraform)
	}

	resp, err := server.PollerResponderNext(beginExportTerraform, req)
	if err != nil {
		return nil, err
	}

	if !contains([]int{http.StatusOK, http.StatusAccepted}, resp.StatusCode) {
		t.beginExportTerraform.remove(req)
		return nil, &nonRetriableError{fmt.Errorf("unexpected status code %d. acceptable values are http.StatusOK, http.StatusAccepted", resp.StatusCode)}
	}
	if !server.PollerResponderMore(beginExportTerraform) {
		t.beginExportTerraform.remove(req)
	}

	return resp, nil
}

// set this to conditionally intercept incoming requests to TerraformServerTransport
var terraformServerTransportInterceptor interface {
	// Do returns true if the server transport should use the returned response/error
	Do(*http.Request) (*http.Response, error, bool)
}
