//go:build go1.18
// +build go1.18

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator. DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package fake

import (
	"errors"
	"fmt"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
	"net/http"
	"strings"
	"sync"
)

// ServerFactory is a fake server for instances of the armworkloads.ClientFactory type.
type ServerFactory struct {
	Server                              Server
	MonitorsServer                      MonitorsServer
	OperationsServer                    OperationsServer
	ProviderInstancesServer             ProviderInstancesServer
	SAPApplicationServerInstancesServer SAPApplicationServerInstancesServer
	SAPCentralInstancesServer           SAPCentralInstancesServer
	SAPDatabaseInstancesServer          SAPDatabaseInstancesServer
	SAPVirtualInstancesServer           SAPVirtualInstancesServer
	SapLandscapeMonitorServer           SapLandscapeMonitorServer
}

// NewServerFactoryTransport creates a new instance of ServerFactoryTransport with the provided implementation.
// The returned ServerFactoryTransport instance is connected to an instance of armworkloads.ClientFactory via the
// azcore.ClientOptions.Transporter field in the client's constructor parameters.
func NewServerFactoryTransport(srv *ServerFactory) *ServerFactoryTransport {
	return &ServerFactoryTransport{
		srv: srv,
	}
}

// ServerFactoryTransport connects instances of armworkloads.ClientFactory to instances of ServerFactory.
// Don't use this type directly, use NewServerFactoryTransport instead.
type ServerFactoryTransport struct {
	srv                                   *ServerFactory
	trMu                                  sync.Mutex
	trServer                              *ServerTransport
	trMonitorsServer                      *MonitorsServerTransport
	trOperationsServer                    *OperationsServerTransport
	trProviderInstancesServer             *ProviderInstancesServerTransport
	trSAPApplicationServerInstancesServer *SAPApplicationServerInstancesServerTransport
	trSAPCentralInstancesServer           *SAPCentralInstancesServerTransport
	trSAPDatabaseInstancesServer          *SAPDatabaseInstancesServerTransport
	trSAPVirtualInstancesServer           *SAPVirtualInstancesServerTransport
	trSapLandscapeMonitorServer           *SapLandscapeMonitorServerTransport
}

// Do implements the policy.Transporter interface for ServerFactoryTransport.
func (s *ServerFactoryTransport) Do(req *http.Request) (*http.Response, error) {
	rawMethod := req.Context().Value(runtime.CtxAPINameKey{})
	method, ok := rawMethod.(string)
	if !ok {
		return nil, nonRetriableError{errors.New("unable to dispatch request, missing value for CtxAPINameKey")}
	}

	client := method[:strings.Index(method, ".")]
	var resp *http.Response
	var err error

	switch client {
	case "Client":
		initServer(s, &s.trServer, func() *ServerTransport { return NewServerTransport(&s.srv.Server) })
		resp, err = s.trServer.Do(req)
	case "MonitorsClient":
		initServer(s, &s.trMonitorsServer, func() *MonitorsServerTransport { return NewMonitorsServerTransport(&s.srv.MonitorsServer) })
		resp, err = s.trMonitorsServer.Do(req)
	case "OperationsClient":
		initServer(s, &s.trOperationsServer, func() *OperationsServerTransport { return NewOperationsServerTransport(&s.srv.OperationsServer) })
		resp, err = s.trOperationsServer.Do(req)
	case "ProviderInstancesClient":
		initServer(s, &s.trProviderInstancesServer, func() *ProviderInstancesServerTransport {
			return NewProviderInstancesServerTransport(&s.srv.ProviderInstancesServer)
		})
		resp, err = s.trProviderInstancesServer.Do(req)
	case "SAPApplicationServerInstancesClient":
		initServer(s, &s.trSAPApplicationServerInstancesServer, func() *SAPApplicationServerInstancesServerTransport {
			return NewSAPApplicationServerInstancesServerTransport(&s.srv.SAPApplicationServerInstancesServer)
		})
		resp, err = s.trSAPApplicationServerInstancesServer.Do(req)
	case "SAPCentralInstancesClient":
		initServer(s, &s.trSAPCentralInstancesServer, func() *SAPCentralInstancesServerTransport {
			return NewSAPCentralInstancesServerTransport(&s.srv.SAPCentralInstancesServer)
		})
		resp, err = s.trSAPCentralInstancesServer.Do(req)
	case "SAPDatabaseInstancesClient":
		initServer(s, &s.trSAPDatabaseInstancesServer, func() *SAPDatabaseInstancesServerTransport {
			return NewSAPDatabaseInstancesServerTransport(&s.srv.SAPDatabaseInstancesServer)
		})
		resp, err = s.trSAPDatabaseInstancesServer.Do(req)
	case "SAPVirtualInstancesClient":
		initServer(s, &s.trSAPVirtualInstancesServer, func() *SAPVirtualInstancesServerTransport {
			return NewSAPVirtualInstancesServerTransport(&s.srv.SAPVirtualInstancesServer)
		})
		resp, err = s.trSAPVirtualInstancesServer.Do(req)
	case "SapLandscapeMonitorClient":
		initServer(s, &s.trSapLandscapeMonitorServer, func() *SapLandscapeMonitorServerTransport {
			return NewSapLandscapeMonitorServerTransport(&s.srv.SapLandscapeMonitorServer)
		})
		resp, err = s.trSapLandscapeMonitorServer.Do(req)
	default:
		err = fmt.Errorf("unhandled client %s", client)
	}

	if err != nil {
		return nil, err
	}

	return resp, nil
}

func initServer[T any](s *ServerFactoryTransport, dst **T, src func() *T) {
	s.trMu.Lock()
	if *dst == nil {
		*dst = src()
	}
	s.trMu.Unlock()
}
